"""! @file filters.py
@brief Filter library for the application
@author xlanro00
"""

import numpy as np
import matplotlib.pyplot as plt
import cv2 as cv


# Parent class for all the filters
class filter:
    def __init__(self, input_file, output_file, img):
        self.input_file = input_file
        self.output_file = output_file
        self.img = img

# Used when no other filters are present, plots original image, 
# may not be necessary
class filter_none(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        plt.imshow(self.img)
        print("No filter selected")


class average(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = np.ones((5, 5), np.float32) / 25
        self.img = cv.filter2D(self.img, -1, kernel)
        plt.imshow(self.img)
        print("Applying average filter")


class blur(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        self.img = cv.blur(self.img, (5, 5))
        plt.imshow(self.img)
        print("Applying normal blur")


class gaussian(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        self.img = cv.GaussianBlur(self.img, (5, 5), 0)
        plt.imshow(self.img)
        print("Applying Gaussian blur")


class median(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        self.img = cv.medianBlur(self.img, 5)
        plt.imshow(self.img)
        print("Applying median blur")


class erode(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = np.ones((5, 5), np.uint8)
        self.img = cv.erode(self.img, kernel, 1)
        plt.imshow(self.img)
        print("Applying erosion")


class dilate(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = np.ones((5, 5), np.uint8)
        self.img = cv.dilate(self.img, kernel, 1)
        plt.imshow(self.img)
        print("Applying dilatation")


# Erosion, then dilatation
class opening(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = cv.getStructuringElement(cv.MORPH_RECT, (3, 3))
        self.img = cv.morphologyEx(self.img, cv.MORPH_CLOSE, kernel)
        plt.imshow(self.img)
        print("Applying opening")


# Dilatation, then erosion
class closing(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = cv.getStructuringElement(cv.MORPH_RECT, (3, 3))
        self.img = cv.morphologyEx(self.img, cv.MORPH_OPEN, kernel)
        plt.imshow(self.img)
        print("Applying closing")
